import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/foundation.dart';
import 'package:path_provider/path_provider.dart';

import '../../abg_utils.dart';

List<ProductData> product = [];
late DateTime _last;

Future<String?> initService(String listen, String providerId) async{

  product = [];
  dprint("initService get from 'service' where $listen arrayContains $providerId");
  await _getLocal();
  bool _localLoad = false;
  if (product.isEmpty) {
    late QuerySnapshot<Map<String, dynamic>> querySnapshot;
    if (listen == "all")
       querySnapshot = await FirebaseFirestore.instance.collection("service").get();
    else
       querySnapshot = await FirebaseFirestore.instance.collection("service")
         .where(listen, arrayContains: providerId).get();

    for (var result in querySnapshot.docs) {
      var _data = result.data();
      var t = ProductData.fromJson(result.id, _data);
      if (!t.delete)
        product.add(t);
    }

    addStat("product", querySnapshot.docs.length);
  }else
    _localLoad = true;

  product.sort((a, b) => b.timeModify.compareTo(a.timeModify));
  if (product.isEmpty)
    _last = DateTime.now();
  else
    _last = product[0].timeModify; //timeUtc;

  if (!_localLoad)
    _saveLocal(_last);

  dprint("services init count=${product.length}");
  return await _listen(listen, providerId);
}

// ignore: cancel_subscriptions
StreamSubscription? listenProductsStream;

Future<String?> _listen(String listen, String providerId) async{
  try{
    work(querySnapshot){
      for (var result in querySnapshot.docs) {
        var _data = ProductData.fromJson(result.id, result.data());

        for (var item in product)
          if (item.id == _data.id){
            dprint("Remove service with id = ${item.id}");
            product.remove(item);
            break;
          }
        dprint("Add service with id = ${_data.id}");
        if (!_data.delete)
          product.add(_data);
      }
      redrawMainWindow();

      product.sort((a, b) => b.timeModify.compareTo(a.timeModify));
      if (product.isEmpty)
        _last = DateTime.now();
      else
        _last = product[0].timeModify; //timeUtc;
      _saveLocal(_last);

      addStat("product listen", querySnapshot.docs.length);
    }

    if (listen == "all"){
      listenProductsStream = FirebaseFirestore.instance.collection("service")
          .where("timeModify", isGreaterThan: _last)
          .snapshots().listen((querySnapshot) {
        work(querySnapshot);
      }, onError: (Object? obj){
        messageError(buildContext, obj.toString());
      });
    }else{
      dprint("products _listen $listen arrayContains $providerId timeModify $_last");
      listenProductsStream = FirebaseFirestore.instance.collection("service")
          .where(listen, arrayContains: providerId).where("timeModify", isGreaterThan: _last)
          .snapshots().listen((querySnapshot) {
        work(querySnapshot);
      }, onError: (Object? obj){
        messageError(buildContext, "Configure Indexes for services");
        dprint("Configure Indexes for services");
      });
    }

  }catch(ex){
    return "listenService " + ex.toString();
  }
  return null;
}

_saveLocal(DateTime _lastTime) async {
  if (kIsWeb){
    localStorage.update(
      "products.json",
          (val) {
        return json.encode({"data": product.map((i) {
          return i.toJson(local: true);
        }).toList()});
      },
      ifAbsent: () {
        return json.encode({"data": product.map((i) {
          return i.toJson(local: true);
        }).toList()});
      },
    );
  }else {
    var directory = await getApplicationDocumentsDirectory();
    var directoryPath = directory.path;
    await File('$directoryPath/products.json')
        .writeAsString(json.encode({"data": product.map((i) {
      var m = i.toJson(local: true);
      // dprint("$m");
      return m;
    }).toList()}));
  }
}

_getLocal() async {
  work(_data){
    if (_data['data'] != null){
      for (var element in List.from(_data['data'])) {
        // dprint(element.toString());
        // dprint(element["id"]);

        var _newItem = ProductData.fromJson(element["id"], element, local: true);

        //
        // проверка на повторения
        // если повторения найдены. Надо очистить кеш и загрузить заново
        //
        for (var item in product)
          if (item.id == _newItem.id) {
            product = [];
            return;
          }

        product.add(_newItem);
      }
      addStat("(CACHE) product load", product.length);
    }
  }

  if (kIsWeb) {
    MapEntry<String, String>? data;
    try {
      data = localStorage.entries.firstWhere((i) => i.key == "products.json");
    } on StateError {
      data = null;
    }
    if (data != null) {
      var _data = json.decode(data.value) as Map<String, dynamic>;
      work(_data);
    }
  }else{
    try{
      var directory = await getApplicationDocumentsDirectory();
      var directoryPath = directory.path;
      var _file = File('$directoryPath/products.json');
      if (await _file.exists()){
        final contents = await _file.readAsString();
        var _data = json.decode(contents);
        work(_data);
      }
    }catch(ex){
      dprint("exception _getLocal $ex");
    }
  }
}